@@ -12,7 +12,6 @@ import os
 import jwt
 from . import urltools
 import logging
-from six import string_types
 
 LOGGER = logging.getLogger("scitokens")
 import uuid
@@ -123,7 +122,7 @@ class SciToken(object):
         :param str issuer: A string indicating the issuer for the token.  It should be an HTTPS address,
                            as specified in https://tools.ietf.org/html/draft-ietf-oauth-discovery-07
         :param int lifetime: Number of seconds that the token should be valid
-        :return str: base64 encoded token
+        :return bytes: base64 encoded token
         """
 
         if include_key is not False:
@@ -164,6 +163,8 @@ class SciToken(object):
             encoded = jwt.encode(payload, self._key, algorithm = self._key_alg, headers={'kid': self._key_id})
         else:
             encoded = jwt.encode(payload, self._key, algorithm = self._key_alg)
+        if isinstance(encoded, bytes):  # pyjwt < 2 returns bytes
+            encoded = encoded.decode("utf-8")
         self._serialized_token = encoded
 
         # Move claims over to verified claims
@@ -175,7 +176,7 @@ class SciToken(object):
 
         # Encode the returned string for backwards compatibility.
         # Previous versions of PyJWT returned bytes
-        return str.encode(encoded)
+        return str.encode(encoded, encoding="utf-8")
 
     def update_claims(self, claims):
         """
@@ -261,7 +262,8 @@ class SciToken(object):
         Verifies the claims pass the current set of validation scripts.
         
         :param str serialized_token: The serialized token.
-        :param str audience: The audience URI that this principle is claiming.  Default: None
+        :param str audience: (Legacy, not checked) The audience URI that this principle is claiming.  Default: None.
+                             Audience is not checked no matter the value.
         :param bool require_key: When True, require the key
         :param bool insecure: When True, allow insecure methods to verify the issuer,
                               including allowing "localhost" issuer (useful in testing).  Default=False
@@ -285,21 +287,24 @@ class SciToken(object):
 
         unverified_headers = jwt.get_unverified_header(serialized_jwt)
         unverified_payload = jwt.decode(serialized_jwt, algorithms=['RS256', 'ES256'],
-                                        options={"verify_signature": False})
+                                        audience=audience,
+                                        options={"verify_signature": False,
+                                                 "verify_aud": False})
         
         # Get the public key from the issuer
         keycache = KeyCache.KeyCache().getinstance()
         if public_key == None:
+            if 'iss' not in unverified_payload:
+                raise MissingIssuerException('Issuer not provided')
+
             issuer_public_key = keycache.getkeyinfo(unverified_payload['iss'],
                                 key_id=unverified_headers['kid'] if 'kid' in unverified_headers else None,
                                 insecure=insecure)
         else:
             issuer_public_key = load_pem_public_key(public_key, backend=backends.default_backend())
         
-        if audience:
-            claims = jwt.decode(serialized_token, issuer_public_key, audience = audience, algorithms=['RS256', 'ES256'])
-        else:
-            claims = jwt.decode(serialized_token, issuer_public_key, algorithms=['RS256', 'ES256'])
+        claims = jwt.decode(serialized_token, issuer_public_key, algorithms=['RS256', 'ES256'],
+                            options={"verify_aud": False})
 
         to_return = SciToken()
         to_return._verified_claims = claims
@@ -345,7 +350,9 @@ class SciToken(object):
                 return SciToken.deserialize(token_data,
                                             audience, require_key, insecure, public_key)
 
-        raise IOError
+        raise OSError(
+            "failed to identify a valid bearer token",
+        )
 
 
 class ValidationFailure(Exception):
@@ -593,9 +600,26 @@ class Enforcer(object):
             return False
         elif value == "ANY":
             return True
-        elif isinstance(self._audience, list):
-            return value in self._audience
-        return value == self._audience
+        
+        # Convert the value and self._audience both to sets
+        # Then perform set intersection
+        values = []
+        if not isinstance(value, list):
+            values = [value]
+        else:
+            values = value
+        set_value = set(values)
+        audiences = []
+        if not isinstance(self._audience, list):
+            audiences = [self._audience]
+        else:
+            audiences = self._audience
+        set_aud = set(audiences)
+        if len(set_value.intersection(set_aud)) > 0:
+            return True
+        else:
+            return False
+
 
     def _validate_ver(self, value):
         if value in self._versions_understood:
@@ -675,7 +699,7 @@ class Enforcer(object):
             return True
 
     def _validate_scope(self, value):
-        if not isinstance(value, string_types):
+        if not isinstance(value, str):
             raise InvalidAuthorizationResource("Scope is invalid.  Must be a space separated string")
         if self._test_access:
             if not self._test_path:
