@@ -7,6 +7,7 @@ import sys
 import unittest
 import tempfile
 import shutil
+import datetime
 
 # Allow unittests to be run from within the project base.
 if os.path.exists("src"):
@@ -14,6 +15,7 @@ if os.path.exists("src"):
 if os.path.exists("../src"):
     sys.path.append("../src")
 
+import jwt
 import scitokens
 from cryptography.hazmat.primitives.asymmetric.rsa import generate_private_key
 import cryptography.hazmat.primitives.asymmetric.ec as ec
@@ -150,15 +152,33 @@ class TestCreation(unittest.TestCase):
 
         serialized_token = token.serialize(issuer = 'local')
 
-        with self.assertRaises(InvalidAudienceError):
-            scitokens.SciToken.deserialize(serialized_token, public_key = self._public_pem, insecure = True)
-
         new_token = scitokens.SciToken.deserialize(serialized_token,
                                                    public_key = self._public_pem,
                                                    insecure = True,
                                                    audience = 'local')
         self.assertIsInstance(new_token, scitokens.SciToken)
 
+
+    def test_any_aud(self):
+        """
+        Test the audience argument with a ANY token to deserialize
+
+        A token with ANY audience should be accepted by all audiences.
+        """
+        token = scitokens.SciToken(key = self._private_key)
+        token.update_claims({'aud': 'ANY'})
+
+        serialized_token = token.serialize(issuer = 'ANY')
+
+        #with self.assertRaises(InvalidAudienceError):
+        #    scitokens.SciToken.deserialize(serialized_token, public_key = self._public_pem, insecure = True)
+
+        new_token = scitokens.SciToken.deserialize(serialized_token,
+                                                   public_key = self._public_pem,
+                                                   insecure = True,
+                                                   audience = 'https://doesnotexist.edu')
+        self.assertIsInstance(new_token, scitokens.SciToken)
+
     def test_serialize(self):
         """
         Test various edge cases of serialization, particularly around failures.
@@ -288,7 +308,7 @@ class TestCreation(unittest.TestCase):
 
         # Test when we give an unsupported algorithm
         unsupported_private_key = ec.generate_private_key(
-            ec.SECP192R1(), default_backend()
+            ec.SECP521R1(), default_backend()
         )
         with self.assertRaises(scitokens.scitokens.UnsupportedKeyException):
             token = scitokens.SciToken(key = unsupported_private_key)
@@ -324,7 +344,7 @@ class TestCreation(unittest.TestCase):
         (bt_fd, bt_tmp) = tempfile.mkstemp()
         os.close(bt_fd)
         if os.path.isfile(bt_path):
-            os.rename(bt_path, bt_tmp)
+            shutil.move(bt_path, bt_tmp)
 
         # check that the function fails properly
         with self.assertRaises(IOError):
@@ -399,7 +419,30 @@ class TestCreation(unittest.TestCase):
         os.remove(bearer_token_file)
         os.remove(bt_path)
         if os.path.isfile(bt_tmp):
-            os.rename(bt_tmp, bt_path)
+            shutil.move(bt_tmp, bt_path)
+
+    def test_noiss(self):
+        """
+        Verify that tokens without the `iss` are not accepted.
+        """
+        encoded_jwt = jwt.encode(
+            {
+                "ver": "scitoken:2.0",
+                "aud": "https://demo.scitokens.org",
+                "exp": datetime.datetime.now(tz=datetime.timezone.utc) + datetime.timedelta(seconds=30),
+                "iat": datetime.datetime.now(tz=datetime.timezone.utc),
+                "nbf": datetime.datetime.now(tz=datetime.timezone.utc),
+                "jti": "eab04181-b63a-42aa-b77d-804742829fc5",
+                "kid": "key-rs256"
+            },
+            "secret",
+            algorithm="HS256",
+            headers={"typ": "JWT", "alg": "HS256", "kid": "key-hs256"}
+        )
+
+        # Test when we give it without issuer
+        with self.assertRaises(scitokens.scitokens.MissingIssuerException):
+           scitokens.SciToken.deserialize(encoded_jwt)
 
 
 if __name__ == '__main__':
